"""The module that defines the ``CommentBase`` model.

SPDX-License-Identifier: AGPL-3.0-only OR BSD-3-Clause-Clear
"""
import typing as t
from dataclasses import dataclass, field

import cg_request_args as rqa

from .. import parsers
from ..utils import to_dict
from .comment_reply import CommentReply


@dataclass
class CommentBase:
    """A comment thread, this contains many replies which contain the actual
    feedback.
    """

    #: The id of the comment base.
    id: "int"
    #: The line on which the comment was placed.
    line: "int"
    #: The id of the file on which this comment was placed.
    file_id: "str"
    #: The id of the work that this comment was placed on. This work will
    #: always contain the file with `file_id`.
    work_id: "int"
    #: The replies, that you are allowed to see, in this comment base.
    replies: "t.Sequence[CommentReply]"

    raw_data: t.Optional[t.Dict[str, t.Any]] = field(init=False, repr=False)

    data_parser: t.ClassVar = rqa.Lazy(
        lambda: rqa.FixedMapping(
            rqa.RequiredArgument(
                "id",
                rqa.SimpleValue.int,
                doc="The id of the comment base.",
            ),
            rqa.RequiredArgument(
                "line",
                rqa.SimpleValue.int,
                doc="The line on which the comment was placed.",
            ),
            rqa.RequiredArgument(
                "file_id",
                rqa.SimpleValue.str,
                doc="The id of the file on which this comment was placed.",
            ),
            rqa.RequiredArgument(
                "work_id",
                rqa.SimpleValue.int,
                doc=(
                    "The id of the work that this comment was placed on. This"
                    " work will always contain the file with `file_id`."
                ),
            ),
            rqa.RequiredArgument(
                "replies",
                rqa.List(parsers.ParserFor.make(CommentReply)),
                doc=(
                    "The replies, that you are allowed to see, in this comment"
                    " base."
                ),
            ),
        ).use_readable_describe(True)
    )

    def to_dict(self) -> t.Dict[str, t.Any]:
        res: t.Dict[str, t.Any] = {
            "id": to_dict(self.id),
            "line": to_dict(self.line),
            "file_id": to_dict(self.file_id),
            "work_id": to_dict(self.work_id),
            "replies": to_dict(self.replies),
        }
        return res

    @classmethod
    def from_dict(
        cls: t.Type["CommentBase"], d: t.Dict[str, t.Any]
    ) -> "CommentBase":
        parsed = cls.data_parser.try_parse(d)

        res = cls(
            id=parsed.id,
            line=parsed.line,
            file_id=parsed.file_id,
            work_id=parsed.work_id,
            replies=parsed.replies,
        )
        res.raw_data = d
        return res
